from apispec_helper.basic_type.common_type import CommonType
from apispec_helper.basic_type.external_documentation import ExternalDocumentation
from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from apispec_helper.basic_type.examples import ExamplesType


@dataclass
class Header(DataclassHelperBase):
    schema: CommonType = None
    content: dict[str, str] = None
    description: str = None
    externalDocs: ExternalDocumentation = None
    required: bool = None
    deprecated: bool = None
    allowEmptyValue: bool = None

    style: str = None
    explode: bool = None
    allowReserved: bool = None

    example: dict = None
    examples: ExamplesType = None



from apispec_helper._internal_utils.post_init_base import PostInitBase
from dataclasses import fields, dataclass
from apispec_helper.utils import to_dict


keyword_fields = ["in_", "type_", "format_"]

@dataclass
class DataclassHelperBase(PostInitBase, dict):
    def __post_init__(self):
        super().__post_init__()

        argument_dict = self.__create_argument_dict()
        dict.__init__(self, **argument_dict)

    def __create_argument_dict(self):
        # dataclass.fields allow us to get self-defined fields. Pre-defined fields won't be included
        dataclass_fields = fields(self)
        argument_dict = dict()

        for single_field in dataclass_fields:
            field_value = getattr(self, single_field.name)
            if field_value is not None:
                argument_dict[self.__rename_keyword_fields(single_field.name)] = getattr(self, single_field.name)

        return argument_dict

    @staticmethod
    def __rename_keyword_fields(field_name):
        return field_name.rstrip("_") if field_name in keyword_fields else field_name

    def to_dict(self):
        return to_dict(self)



from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from typing import List


@dataclass
class ServerVariables(DataclassHelperBase):
    enum_: List[str]
    default: str
    description: str = None


@dataclass
class Server(DataclassHelperBase):
    url: str
    description: str = None
    variables: dict[str, ServerVariables] = None



from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from apispec_helper.basic_type.server import Server


@dataclass
class Link(DataclassHelperBase):
    operationRef: str = None
    operationId: str = None

    parameters: dict[str, str] = None
    requestBody: str = None
    description: str = None
    server: Server = None



from apispec_helper.path.path_item import PathItem


class PathBase:
    def __init__(self, path: str, path_item_definition: PathItem):
        self.__path_item_definition = path_item_definition
        self.__path = path

    @property
    def operations(self):
        return self.__path_item_definition.to_dict()['operations']

    @property
    def summary(self):
        return self.__path_item_definition.to_dict()['summary']

    @property
    def description(self):
        return self.__path_item_definition.to_dict()['description']

    @property
    def servers(self):
        return self.__path_item_definition.to_dict()['servers']

    @property
    def parameters(self):
        return self.__path_item_definition.to_dict()['parameters']

    @property
    def path(self):
        return self.__path

    @property
    def apispec_parameter(self) -> dict:
        parameter_list = ["path", "operations", "summary", "description", "servers", "parameters"]
        parameter_dict = dict()

        for parameter in parameter_list:
            try:
                value = getattr(self, parameter)
                parameter_dict[parameter] = value
            except KeyError:
                pass

        return parameter_dict



from apispec_helper.component.component_base import ComponentBase

from apispec_helper.component.content import ContentType
from apispec_helper.component.content import Encoding, MediaType
from apispec_helper.component.content import CommonMediaTypeName

from apispec_helper.component.header import Header

from apispec_helper.component.link import Link

from apispec_helper.component.parameter import ParameterType
from apispec_helper.component.parameter import Parameter
from apispec_helper.component.parameter import ParameterLocation
from apispec_helper.component.parameter_style import ParameterStyle

from apispec_helper.component.request_body import RequestBody

from apispec_helper.component.response import Response

from apispec_helper.component.security_scheme import SecurityScheme
from apispec_helper.component.security_scheme import HTTPAuthenticationScheme, APIKeyLocation, SecuritySchemeType
from apispec_helper.component.security_scheme import OAuthFlows, OAuthFlow

__all__ = [
    "ComponentBase",

    "ContentType","Encoding", "MediaType","CommonMediaTypeName",
    
    "Header",

    "Link",

    "ParameterType", "Parameter", "ParameterLocation", "ParameterStyle",
    
    "RequestBody",

    "Response",

    "SecurityScheme", "HTTPAuthenticationScheme", "APIKeyLocation", "SecuritySchemeType", "OAuthFlows", "OAuthFlow"
]


from dataclasses import dataclass
from apispec_helper.basic_type.common_type import CommonType
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from apispec_helper.component.header import Header
from typing import Union


class CommonMediaTypeName:
    APPLICATION_JSON = "application/json"
    APPLICATION_OCTET_STEAM = "application/octet-stream"
    APPLICATION_X_WWW_FORM_URLENCODED = "application/x-www-form-urlencoded"

    MULTIPART_FORM_DATA = "multipart/form-data"
    MULTIPART_MIXED = "multipart/mixed"

    TEXT_PAIN = "text/plain"


@dataclass
class Encoding(DataclassHelperBase):
    contentType: str

    # dict[str: str] for reference
    headers: dict[str, Header] | dict[str, str] = None
    # reference to ParameterStyle
    style: str = None
    explode: bool = None
    allowReserved: bool = None


@dataclass
class MediaType(DataclassHelperBase):
    schema: CommonType
    encoding: dict[str, Encoding] = None
    example: dict = None
    examples: dict = None


ContentType = Union[dict[str, MediaType]]



from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from apispec_helper._internal_utils.post_init_base import PostInitBase
from apispec_helper._internal_utils.mutual_exclusive_field_checker import MutualExclusiveFieldChecker
from apispec_helper._internal_utils.one_of_field_checker import OneOfFieldChecker


class ExampleDataclassHelperBase(PostInitBase):
    def __post_init__(self):
        super().__post_init__()
        MutualExclusiveFieldChecker(self, ["value", "externalValue"]).execute()
        OneOfFieldChecker(self, ["value", "externalValue"]).execute()


@dataclass
class Example(DataclassHelperBase, ExampleDataclassHelperBase):
    summary: str = None
    description: str = None

    value: int | str | dict = None
    externalValue: str = None


ExamplesType = dict[str: Example]
